/*
 * Copyright (C) 2004-2005 ruby-ecore team (see AUTHORS)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <Ecore_Evas.h>

#include "../ecore/rb_ecore.h"
#include "rb_ecore_evas_main.h"
#include "rb_ecore_evas.h"

static void c_free (RbEcoreEvas *ee)
{
	c_ecore_evas_free (ee, true);
}

static VALUE c_alloc (VALUE klass)
{
	RbEcoreEvas *ee = NULL;

	ecore_init ();
	ecore_evas_init ();

	return Data_Make_Struct (klass, RbEcoreEvas,
	                         c_ecore_evas_mark, c_free, ee);
}

/*
 * call-seq:
 *  Ecore::Evas::Buffer.new([w, h]) => buffer
 *
 * Creates an Ecore::Evas::Buffer object.
 */
static VALUE c_init (int argc, VALUE *argv, VALUE self)
{
	VALUE w, h;
	int iw = 0, ih = 0;

	GET_OBJ (self, RbEcoreEvas, ee);

	rb_scan_args (argc, argv, "02", &w, &h);

	if (!NIL_P (w)) {
		Check_Type (w, T_FIXNUM);
		iw = FIX2INT (w);
	}

	if (!NIL_P (h)) {
		Check_Type (h, T_FIXNUM);
		ih = FIX2INT (h);
	}

	ee->real = ecore_evas_buffer_new (iw, ih);

	rb_call_super (argc, argv);

	return self;
}

static VALUE c_pixels_get (VALUE self)
{
	const int *p;
	int w = 0, h = 0;

	GET_OBJ (self, RbEcoreEvas, ee);

	p = ecore_evas_buffer_pixels_get (ee->real);
	if (!p)
		return rb_str_new2 (""); /* FIXME raise an error instead? */

	ecore_evas_geometry_get (ee->real, NULL, NULL, &w, &h);

	return rb_str_new ((char *) p, h * w * 4);
}

void Init_Buffer (void)
{
	VALUE c = rb_define_class_under (mEvas, "Buffer", cEcoreEvas);

	rb_define_alloc_func (c, c_alloc);
	rb_define_method (c, "initialize", c_init, -1);
	rb_define_method (c, "pixels", c_pixels_get, 0);
}
